package com.savagehomeautomation.devoxx;

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

import com.google.gson.Gson;
import com.pi4j.component.sensor.Sensor;
import com.pi4j.component.sensor.SensorListener;
import com.pi4j.component.sensor.SensorStateChangeEvent;
import com.pi4j.component.sensor.impl.GpioSensorComponent;
import com.pi4j.io.gpio.GpioController;
import com.pi4j.io.gpio.GpioFactory;
import com.pi4j.io.gpio.GpioPinDigitalInput;
import com.pi4j.io.gpio.PinPullResistance;
import com.pi4j.io.gpio.RaspiPin;
import javafx.application.Application;
import javafx.fxml.FXMLLoader;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.stage.Stage;
import org.eclipse.paho.client.mqttv3.IMqttActionListener;
import org.eclipse.paho.client.mqttv3.IMqttToken;
import org.eclipse.paho.client.mqttv3.MqttAsyncClient;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.MqttMessage;

/**
 * @author Robert Savage 
 * @see http://www.pi4j.com 
 * @see http://www.savagehomeautomation.com
 * @see https://twitter.com/savageautomate
 * 
 * This is the main class that load the correct JavaFX scene and get this 
 * program started.
 */
public class Main_For_Raspberry_Pi extends Application {
    
    // -----------------------------------------------------------------------
    // NOTES FOR USING JAVA.FX ON THE RASPBERRY PI:
    // -----------------------------------------------------------------------
    // Set the memory split on the Raspberry Pi to include at least 128 MB for video.
    //      [-- raspi-config :: under advanced settings --]
    // Set the frame buffer on the Raspberry Pi to 720p (1280x720) resolution.
    //      [-- sudo nano /boot/config.txt --]
    //      [-- "framebuffer_width=1280" :: "framebuffer_height=720" --]
    // Set the HDMI mode on the Raspberry Pi to 720p.  
    //      [-- sudo nano /boot/config.txt --]
    //      [-- "hdmi_group=1" (CEA) :: "hdmi_mode=4" (720p 60 Hz) --]
    // Install Java 8 JDK on the Raspberry Pi.
    //      [-- sudo apt-get update --]
    //      [-- sudo apt-get install oracle-java8-jdk --]
    // -----------------------------------------------------------------------
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        launch(args);
    }
    
    @Override
    public void start(Stage stage) throws Exception {
        
        // load JavaFX app
        final Parent root = FXMLLoader.load(getClass().getResource("/fxml/Main.fxml"));
        Scene scene = new Scene(root);
        stage.setScene(scene);
        stage.setMaximized(true);
        stage.show();
        
        // create an connect a MQTT async client
        MqttAsyncClient mqtt = MqttHelper.createAsync();

        // create a sensors state object to track and report sensor status changes
        SensorStates sensorStates = new SensorStates();
        
        // create Pi4J GPIO controller for the Raspberry Pi
        GpioController gpio = GpioFactory.getInstance();
        
        // ---------------------------------------------------------------------------
        // (DRIVEWAY SENSOR) 
        // ---------------------------------------------------------------------------
        // We are using a Seco-larm Enforcer E-931-S35RRQ Infrared Beam sensor 
        // to detect any activity in out private driveway.  When the continious 
        // beam is broken by a vehicle or person walking up the driveway the sensor
        // it will trigger a relay and provide a closed circuit for our GPIO pin.
        // If the IR beam is not disrupted, the sensor will open the circuit.
        // ---------------------------------------------------------------------------

        // provision GPIO pin for use swith the the IR beam sensor (GPIO PIN #4)
        GpioPinDigitalInput drivewaySensorPin = gpio.provisionDigitalInputPin(
                RaspiPin.GPIO_04, PinPullResistance.PULL_DOWN);
        
        // create a sensor instance for the flood detection device 
        Sensor drivewaySensor = new GpioSensorComponent(drivewaySensorPin);
        
        // set default sensor status
        sensorStates.drivewayTripped = drivewaySensor.isClosed();

        // create a sensor listener 
        drivewaySensor.addListener((SensorListener) (SensorStateChangeEvent event) -> {

            // display state change on console
            System.out.println("[Pi4J] --> DRIVEWAY SENSOR EVENT: " + event.getNewState());
            
            // update sensor status
            sensorStates.drivewayTripped = drivewaySensor.isClosed();

            // notify MQTT topic
            publish(mqtt, sensorStates);
        });

        
        // ---------------------------------------------------------------------------
        // (FLOOD/WATER LEVEL SENSOR)  
        // ---------------------------------------------------------------------------
        // We are using a water level sensor to detect a flood condition.
        // If the water level rises to a flood condition it will close the circuit; 
        // ff the water level goes below the threshold level then the sensor will
        // open the circuit.
        // ---------------------------------------------------------------------------
        
        // provision GPIO pin for use swith the the flood sensor (GPIO PIN #3)
        GpioPinDigitalInput floodSensorPin = gpio.provisionDigitalInputPin(
                RaspiPin.GPIO_03, PinPullResistance.PULL_DOWN);       
        
        // create a sensor instance for the flood detection device 
        Sensor floodSensor = new GpioSensorComponent(floodSensorPin);

        // set default sensor status
        sensorStates.floodTripped = floodSensor.isClosed();
        
        // create a sensor listener 
        floodSensor.addListener((SensorListener) (SensorStateChangeEvent event) -> {
            
            // display state change on console
            System.out.println("[Pi4J] --> FLOOD SENSOR EVENT: " + event.getNewState());
            
            // update sensor status
            sensorStates.floodTripped = floodSensor.isClosed();

            // notify MQTT topic
            publish(mqtt, sensorStates);
        });
        
        // ---------------------------------------------------------------------------
        // (HVAC LEAK SENSOR) 
        // ---------------------------------------------------------------------------
        // We are using a Diversitech WetSwitch (WS-1) to detected moisuture 
        // in the HVAC overflow catch basin/pan.  If the WetSwitch detector 
        // senses moisture, it will trigger a relay and provide a closed circuit
        // for our GPIO pin.  If no moisture is detected, then the detector will
        // open the circuit.
        // ---------------------------------------------------------------------------
        
        // provision GPIO pin for use swith the the power sensor (GPIO PIN #1)
        GpioPinDigitalInput hvacSensorPin = gpio.provisionDigitalInputPin(
                RaspiPin.GPIO_01, PinPullResistance.PULL_DOWN);

        // create a sensor instance for the HVAC leak detection device 
        Sensor hvacSensor = new GpioSensorComponent(hvacSensorPin);

        // set default sensor status
        sensorStates.hvacTripped = hvacSensor.isClosed();
        
        // create a sensor listener 
        hvacSensor.addListener((SensorListener) (SensorStateChangeEvent event) -> {
            
            // display state change on console
            System.out.println("[Pi4J] --> HVAC LEAK SENSOR EVENT: " + event.getNewState());
            
            // update sensor status
            sensorStates.hvacTripped = hvacSensor.isClosed();

            // notify MQTT topic
            publish(mqtt, sensorStates);
        });      
        
        
        // ---------------------------------------------------------------------------
        // (MAILBOX SENSOR) 
        // ---------------------------------------------------------------------------
        // We are using a simple magnetic reed security sensor attached to the mailbox 
        // and the magnet attached to the mailbox door to detect when the mailbox 
        // door is opened|closed.  If the mailbox door is closed the magnetic reed 
        // sensor will provide a closed circuit for our GPIO pin.  When the door is 
        // opened, the sensor will open the circuit.
        // ---------------------------------------------------------------------------
        
        // provision GPIO pin for use swith the the mailbox sensor (GPIO PIN #5)
        GpioPinDigitalInput mailboxSensorPin = gpio.provisionDigitalInputPin(
                RaspiPin.GPIO_05, PinPullResistance.PULL_DOWN);
        
        // create a sensor instance for the mailbox door sensor
        Sensor mailboxSensor = new GpioSensorComponent(mailboxSensorPin);

        // set default sensor status
        sensorStates.mailboxTripped = mailboxSensor.isOpen();
        
        // create a sensor listener 
        mailboxSensor.addListener((SensorListener) (SensorStateChangeEvent event) -> {
        
            // display state change on console
            System.out.println("[Pi4J] --> MAILBOX SENSOR EVENT: " + event.getNewState());
            
            // update sensor status
            sensorStates.mailboxTripped = mailboxSensor.isOpen();

            // notify MQTT topic
            publish(mqtt, sensorStates);
        });           
        
        // ---------------------------------------------------------------------------
        // (MQTT CONNECT & SYNC) 
        // ---------------------------------------------------------------------------
        // connect to MQTT server; on connection, publish current sensor states
        MqttHelper.connect(mqtt, new IMqttActionListener() {
            @Override
            public void onSuccess(IMqttToken asyncActionToken) {
                // notify MQTT topic on startup
                publish(mqtt, sensorStates);
            }

            @Override
            public void onFailure(IMqttToken asyncActionToken, Throwable exception) {
            }
        });
    }
    
    protected void publish(MqttAsyncClient mqtt, SensorStates sensorStates){
        try{
            // convert sensors state object into JSON string
            String message = new Gson().toJson(sensorStates);
            
            // create MQTT message from JSON string
            MqttMessage msg = new MqttMessage(message.getBytes());
            msg.setRetained(true);
            msg.setQos(MqttHelper.QOS);
            
            // publish MQTT message
            mqtt.publish(MqttHelper.TOPIC, msg);
            System.out.println("[MQTT] >>> (PUBLISHED) " + message);
        }
        catch(MqttException ex){
            ex.printStackTrace();
        }
    }    
}
